import logging
from os.path import dirname, join

import shutil
import tempfile
from syncloud.app import logger

from syncloud.sam.manager import get_sam
from syncloud.sam.config import SamConfig

test_dir = dirname(__file__)
logger.init(logging.DEBUG, console=True)

class BaseSamTest():

    def setup(self):
        self.config_dir = tempfile.mkdtemp()
        apps_dir = tempfile.mkdtemp()
        status_dir = tempfile.mkdtemp()
        self.releases_dir = tempfile.mkdtemp()

        apps_url_template = 'file://'+self.releases_dir+'/{}.zip'

        self.config = SamConfig(join(self.config_dir, 'sam.cfg'))
        self.config.set_apps_dir(apps_dir)
        self.config.set_status_dir(status_dir)
        self.config.set_apps_url_template(apps_url_template)
        self.config.set_bin_dir(join(test_dir, 'data', 'bin', 'good'))

        self.sam = get_sam(self.config_dir)

    def check_single_application(self, id, name, current_version, installed_version):
        applications = self.sam.list()
        assert applications is not None
        assert len(applications) == 1

        test_app = applications[0]
        assert test_app.app.id == id
        assert test_app.app.name == name
        assert test_app.current_version == current_version
        assert test_app.installed_version == installed_version

    def create_release(self, release, as_release=None):
        test_data_dir = join(test_dir, 'data')
        prepare_dir_root = tempfile.mkdtemp()
        prepare_dir = join(prepare_dir_root, 'app-' + release)

        release_path = join(test_data_dir, release)
        shutil.copytree(release_path, prepare_dir)

        archive = shutil.make_archive(release, 'zip', prepare_dir_root)

        if not as_release:
            as_release = release
        shutil.copy(archive, join(self.releases_dir, as_release + '.zip'))


class TestApplicationAction(BaseSamTest):

    def test_list(self):
        self.create_release('release-1.0')
        self.sam.update('release-1.0')

        self.check_single_application('test-app', 'test app', '1.0', None)

    def test_install(self):
        self.create_release('release-1.0')
        self.sam.update('release-1.0')

        self.sam.install('test-app')

        self.check_single_application('test-app', 'test app', '1.0', '1.0')

    def test_upgrade(self):
        self.create_release('release-1.0')
        self.sam.update('release-1.0')

        self.sam.install('test-app')

        self.create_release('release-1.1')
        self.sam.update('release-1.1')

        self.check_single_application('test-app', 'test app', '1.1', '1.0')

        self.sam.upgrade('test-app')

        self.check_single_application('test-app', 'test app', '1.1', '1.1')

    def test_remove(self):
        self.create_release('release-1.0')
        self.sam.update('release-1.0')

        self.sam.install('test-app')

        self.sam.remove('test-app')

        self.check_single_application('test-app', 'test app', '1.0', None)


    def test_remove_hook_good(self):

        self.config.set_bin_dir(join(test_dir, 'data', 'pre-remove', 'bin', 'good'))
        self.sam = get_sam(self.config_dir)
        self.create_release('release-1.0')
        self.sam.update('release-1.0')

        self.sam.install('test-app')

        self.sam.remove('test-app')

        self.check_single_application('test-app', 'test app', '1.0', None)


    def test_remove_hook_missing(self):
        self.config.set_bin_dir(join(test_dir, 'data', 'pre-remove', 'bin', 'missing'))
        self.sam = get_sam(self.config_dir)
        self.create_release('release-1.0')
        self.sam.update('release-1.0')

        self.sam.install('test-app')

        self.sam.remove('test-app')

        self.check_single_application('test-app', 'test app', '1.0', None)

    def test_remove_hook_bad(self):

        self.config.set_bin_dir(join(test_dir, 'data', 'pre-remove', 'bin', 'bad'))
        self.sam = get_sam(self.config_dir)
        self.create_release('release-1.0')
        self.sam.update('release-1.0')

        self.sam.install('test-app')

        try:
            self.sam.remove('test-app')
            assert False
        except Exception, e:
            assert True

        self.check_single_application('test-app', 'test app', '1.0', '1.0')

    def test_install_hook_bad(self):

        self.config.set_bin_dir(join(test_dir, 'data', 'post-install', 'bin', 'bad'))
        self.sam = get_sam(self.config_dir)
        self.create_release('release-1.0')
        self.sam.update('release-1.0')

        try:
            self.sam.install('test-app')
            assert False
        except Exception, e:
            assert True

        self.check_single_application('test-app', 'test app', '1.0', None)


class TestReleases(BaseSamTest):

    def test_update_same_release(self):

        self.create_release('release-1.0')
        self.sam.update('release-1.0')

        self.check_single_application('test-app', 'test app', '1.0', None)

        self.create_release('release-1.0-1', as_release='release-1.0')
        self.sam.update()

        self.check_single_application('test-app', 'test app', '1.0-1', None)

    def test_update_new_release(self):

        self.create_release('release-1.0')
        self.sam.update('release-1.0')

        self.check_single_application('test-app', 'test app', '1.0', None)

        self.create_release('release-1.1')
        self.sam.update('release-1.1')

        self.check_single_application('test-app', 'test app', '1.1', None)

    def test_update_bootstrap_no_apps_dir(self):
        self.config.set_apps_dir(join(self.config.apps_dir(), 'non_existent'))

        self.sam = get_sam(self.config_dir)

        self.create_release('release-1.0')
        self.sam.update('release-1.0')

        self.check_single_application('test-app', 'test app', '1.0', None)

    def test_update_not_required_app(self):
        self.create_release('release-1.0')
        apps = self.sam.update('release-1.0')

        assert len(apps) == 0

        self.check_single_application('test-app', 'test app', '1.0', None)

    def test_update_usual_app(self):
        self.create_release('release-1.0')
        self.sam.update('release-1.0')

        self.sam.install('test-app')

        self.create_release('release-1.1')
        apps = self.sam.update('release-1.1')

        assert len(apps) == 1
        assert apps[0].app.id == 'test-app'

        self.sam.upgrade_all()

        self.check_single_application('test-app', 'test app', '1.1', '1.1')

    def test_update_required_app(self):
        self.create_release('release-required')
        self.sam.update('release-required')
        self.sam.upgrade_all()

        self.check_single_application('test-app', 'test app', '1.0', '1.0')